package com.persagy.ztkencryptdecodedata.kafka;

import com.alibaba.fastjson.JSONObject;
import com.persagy.ztkencryptdecodedata.dataSafety.DecryptInputMessageService;
import com.persagy.ztkencryptdecodedata.netty.NettyClinet;
import org.apache.kafka.clients.consumer.ConsumerRebalanceListener;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.TopicPartition;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Collection;
import java.util.Properties;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;

@Component
@Order(value = 2)
public class CloudKafkaConsumerFromEdgeTopic implements CommandLineRunner {

    private  static Logger logger = LoggerFactory.getLogger(CloudKafkaConsumerFromEdgeTopic.class.getClass());
    BlockingQueue<String> messageTcpSendQueue = new LinkedBlockingQueue<>(1024 * 1024);
     //   BlockingQueue messageTcpSendQueue =null;

    @Autowired
    private DecryptInputMessageService decryptInputMessageService;

    @Autowired
    private NettyClinet nettyClinet;

    @Value("${spring.kafka.consumer.bootstrap-servers}")
    private  String consumerBootstrap;
    @Value("${spring.kafka.consumer.group-id}")
    private  String groupId;
    @Value("${spring.kafka.consumer.auto-offset-reset}")
    private  String autooffet;
    @Value("${spring.kafka.consumer.fetch-min-size}")
    private  int  fetchMinSize;
    @Value("${spring.kafka.consumer.fetch-max-wait}")
    private  int  fetchMaxWait;
    @Value("${spring.kafka.consumer.enable-auto-commit}")
    private  boolean enableAutoCommit;
    @Value("${spring.kafka.consumer.max-poll-records}")
    private  int maxPollRecords;
    @Value("${spring.kafka.consumer.key-deserializer}")
    private  String keyDeserializer;
    @Value("${spring.kafka.consumer.value-deserializer}")
    private String valueDeserializer;

    @Value("${spring.location}")
    private String location;



    @PostConstruct
    public void init() {
        nettyClinet.setMessageTcpSendQueue(messageTcpSendQueue);
    }

    @Override
    public void run(String... args) throws Exception {
        int i=0;
        System.out.println("Colud_Subscribe_Edge_Update_messages");
        Properties props = new Properties();
        props.put("bootstrap.servers", consumerBootstrap);
        props.put("group.id", groupId);
        props.put("auto.offset.reset", autooffet);
        props.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        props.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");

        //metadata.max.age.ms 1000
        KafkaConsumer<String, String> consumer =  new KafkaConsumer<>(props);

        String fixTopic="";

        if ("Cloud".equals(location)){
            fixTopic="Edge";
        }
        if ("Edge".equals(location)){
            fixTopic ="Cloud";
        }

        Pattern pattern = Pattern.compile(fixTopic+".*");
        consumer.subscribe(pattern, new ConsumerRebalanceListener() {

            @Override
            public void onPartitionsRevoked(Collection<TopicPartition> arg0) {
                // TODO nothing：再均衡监听器
            }

            @Override
            public void onPartitionsAssigned(Collection<TopicPartition> arg0) {
                // TODO nothing：再均衡监听器
            }
        });
        try {

        while (true) {
            /*
             * poll() 方法返回一个记录列表。
             * 每条记录都包含了记录所属主题的信息、记录所在分区的信息、记录在分区里的偏移量，以及记录的键值对。
             * 我们一般会遍历这个列表，逐条处理这些记录。
             * 传给poll() 方法的参数是一个超时时间，用于控制 poll() 方法的阻塞时间（在消费者的缓冲区里没有可用数据时会发生阻塞）。
             * 如果该参数被设为 0，poll() 会立即返回，否则它会在指定的毫秒数内一直等待 broker 返回数据。
             * 而在经过了指定的时间后，即使还是没有获取到数据，poll()也会返回结果。
             */
            ConsumerRecords<String, String> records = consumer.poll(1000);
            for (ConsumerRecord<String, String> record : records) {
                System.out.println(">>>>>>>"+i);
                System.out.println("topic = " + record.topic() + ", partition = " + record.partition()+", messages = "+ JSONObject.parseObject(decryptInputMessageService.DecryptMsgInputMessage(record.value())).toString());
                messageTcpSendQueue.offer(JSONObject.parseObject(decryptInputMessageService.DecryptMsgInputMessage(record.value())).toString(), 1000, TimeUnit.MICROSECONDS);
               // messageTcpSendQueue.add(JSONObject.parseObject(decryptInputMessageService.DecryptMsgInputMessage(record.value())).toString());

            }

        }

        } finally {

        /*
         * 在退出应用程序之前使用 close() 方法关闭消费者。
         * 网络连接和 socket 也会随之关闭，并立即触发一次再均衡，而不是等待群组协调器发现它不再发送心跳并认定它已死亡，
         * 因为那样需要更长的时间，导致整个群组在一段时间内无法读取消息。
         */
        consumer.close();
    }
    }

}
